from flask.testing import FlaskClient
from flask_theme_adminlte3.app import create_app
import pytest

def test_configfile_not_exists(client:FlaskClient):

    with pytest.raises(EnvironmentError) as e_info:
        app = create_app('doesnotexist.py')

def test_configfile_does_load(tmp_path):
    conf = tmp_path / "config.py"
    conf.write_text("""CONFIG_KEY_1='CONFIG_VAL_1'""")
    app = create_app(conf)
    assert app.config['CONFIG_KEY_1']=='CONFIG_VAL_1'

def test_config_kwargs_does_load():
    app = create_app(CONFIG_KEY_2='CONFIG_VAL_2')
    assert app.config['CONFIG_KEY_2']=='CONFIG_VAL_2'



from collections import OrderedDict


class MenuItem(object):
    """Application menu item."""

    TYPE_LINK = "link"
    TYPE_HEADER = "header"
    TYPE_DROPDOWN_DIVIDER = "dropdown divider"
    TYPE_FORM = "form"

    __slots__ = (
        "_active",
        "_children",
        "id",
        "title",
        "url",
        "parent",
        "type",
        "icon",
        "help",
        "badge",
        "_class",
        "form",
    )

    def __init__(
        self,
        id_item,
        title,
        url,
        parent=None,
        item_type=TYPE_LINK,
        icon=False,
        help=None,
        badge=None,
        _class="",
        form=None,
    ):
        """
        Arguments:
            id_item (int|str): the unique identifier of the menu item.
            title (str): item title.
            url (str): the URL that the menu item refers to.
            parent (MenuItem): link to the parent menu item.
            item_type (str): type of menu item, see MenuItem.TYPE_ *.
            icon (str): CSS classes for the icon.
            help (str): hover hint.
            badge (tuple): text label, the first element is text, the second element is style.
        """
        self._active = False
        self._children = []

        self.id = id_item
        self.title = title
        self.url = url
        self.parent = parent
        self.type = item_type
        self.icon = icon
        self.help = help
        self.badge = badge
        self._class = _class
        self.form = form

    def __str__(self):
        return self.title

    def __iter__(self):
        for child in self.children:
            yield child

    def add_badge(self, text, color):
        """Adds a text label to a menu item."""
        self.badge = (text, color)

    def append_child(self, child):
        """Adds a child menu item."""
        child.parent = self
        self._children.append(child)

    @property
    def children(self):
        yield from self._children

    @children.setter
    def children(self, children):
        for child in children:
            self.append_child(child)

    def has_children(self):
        """Returns true if the menu item has a submenu."""
        return bool(self._children)

    def has_parent(self):
        """Returns true if the menu item is a child."""
        return self.parent is not None

    def is_active(self):
        """Returns true if the menu item is selected, otherwise false."""
        return self._active

    def remove_child(self, child):
        # fixme: родитель и исключение
        if child in self.children:
            self._children.remove(child)

    def set_active(self, state):
        """Sets the status of a menu item as active or not."""
        self._active = bool(state)

        if self.has_parent():
            self.parent.set_active(state)


class Menu(object):
    """Application menu."""

    __slots__ = ("_items", "title")

    def __init__(self, title=None):
        self._items = OrderedDict()
        self.title = title

    def __iter__(self):
        for id_item, item in self._items.items():
            if not item.has_parent():
                yield item

    def activate_by_path(self, path):
        """Makes active a menu item whose URL matches the one specified in the argument."""
        for item in self._items.values():
            if item.type == MenuItem.TYPE_LINK and item.url == path:
                item.set_active(True)
                return True
        return False

    def activate_by_function(self, is_active, all=False):
        """Makes active a menu item if the activate function returns True"""
        activated = False
        for item in self._items.values():
            if is_active(item):
                item.set_active(True)
                activated = True
                if not all:
                    return activated
        return activated

    def add_item(self, item: MenuItem):
        """Adds a new menu item."""
        if item.has_parent():
            item.parent.append_child(item)
        self._items[item.id] = item

    def get_item(self, id_item):
        """Returns a menu item with the specified unique identifier."""
        return self._items.get(id_item)



# -*- coding: utf-8 -*-
import os

module_path = os.path.dirname(os.path.abspath(__file__))
tests = [
    f for f in os.listdir(module_path) if f.endswith(".py") and f != "__init__.py"
]
__all__ = tests
print(
    "Imported tests: %s" % ", ".join(tests)
    if tests
    else "No tests avaiable in the tests directory."
)



import pytest
import flask
from flask_theme_adminlte3.app import create_app

@pytest.fixture(scope='function')
def app():

    app = create_app()
    app.testing = True
    yield app

@pytest.fixture()
def runner(app):
    r = app.test_cli_runner()

    def runfunc(command):
        return r.invoke(r.app.cli,command.split())

    yield runfunc



try:
    from flask_assets import Environment, Bundle
except ImportError:
    flask_assets = None
import flask
import pathlib
import gzip
import tempfile
import os

def gz(inp, out, **kw):
    n = ''
    with tempfile.NamedTemporaryFile('wb',delete=False) as t:
        with gzip.open(t.name, "wb") as f:
            f.write(inp.read().encode())
            n = t.name
    with open(n,'rb') as f:
        out.write(f.read().decode('ISO-8859–1'))
    os.unlink(n)

def init_app(app:flask.Flask):
    if not flask_assets:
        app.logger.debug(f"Disabling assets extention since 'flask_assets' module is missing")
        return
    filters = []#[gz]
    assets = Environment()
    bp_name = app.config['THEME_BLUEPRINT_NAME']
    prefix = app.config['THEME_ASSETS_PATH']
    if prefix is None:
        prefix = pathlib.Path(app.root_path) / 'assets'

    styles = Bundle(
        f'{prefix}/plugins/icheck-bootstrap/icheck-bootstrap.css',
        f'{prefix}/dist/css/adminlte.min.css',
        f'{prefix}/plugins/overlayScrollbars/css/OverlayScrollbars.min.css',
        f'{prefix}/plugins/fontawesome-free/css/all.min.css',
        filters=['cssmin']+filters,
        output=f'{bp_name}/css/{bp_name}.css',
    )

    scripts = Bundle(
        f'{prefix}/plugins/jquery/jquery.js',
        f'{prefix}/plugins/jquery-ui/jquery-ui.min.js',
        f'{prefix}/plugins/bootstrap/js/bootstrap.bundle.min.js',
        f'{prefix}/plugins/overlayScrollbars/js/jquery.overlayScrollbars.js',
        f'{prefix}/plugins/overlayScrollbars/js/OverlayScrollbars.js',
        f'{prefix}/dist/js/adminlte.js',
        filters=['jsmin']+filters,
        output=f'{bp_name}/js/{bp_name}.js',
    )

    assets.register(f'{bp_name}_css', styles)
    assets.register(f'{bp_name}_js', scripts)

    assets.init_app(app)



"""
Provides ready-made implementations for filters used in templates.
"""

from string import Template

from .constants import ThemeColor


__all__ = ("humanize", "if_true", "navbar_skin", "sidebar_skin", "replace_with_flag")


def humanize(dt, locale="en_us", time_zone=None):
    """The filter converts the date to human readable."""
    import arrow
    from dateutil import tz

    dt = arrow.get(dt, tz.gettz(time_zone))
    return dt.humanize(locale=locale, only_distance=True)


def if_true(value, replace_with=None):
    """Replaces the value with the passed if the value is true."""
    if not value:
        return ""

    if replace_with is None:
        return value

    return Template(replace_with).safe_substitute(value=value)


def replace_with_flag(locale):
    """The filter replaces the locale with the CSS flag classes of the flag-icon-css library."""
    locale = locale.replace("-", "_").lower().rsplit("_", maxsplit=1)

    if len(locale) == 2:
        return f"flag-icon flag-icon-{locale[-1]}"

    return ""


def navbar_skin(color):
    """Returns a collection of classes to style the navigation bar."""
    if color:
        light = {
            ThemeColor.LIGHT,
            ThemeColor.WARNING,
            ThemeColor.WHITE,
            ThemeColor.ORANGE,
        }
        style = "light" if color in light else f"dark"
        return f"navbar-{style} navbar-{color}"
    return ""


def sidebar_skin(color, light=False):
    """Returns a collection of classes to style the main sidebar bar."""
    if color:
        style = "light" if light else f"dark"
        return f"sidebar-{style}-{color}"
    return ""



import flask, werkzeug
from flask import signals
import pathlib
from flask.signals import Namespace
from .datatypes import Menu, MenuItem
import yaml, os

MENUS_YAML = "menus.yml"
DEFAULT_MENUS_FILE = os.path.join(os.path.dirname(__file__), MENUS_YAML)


class MenuLoader:
    """
    A menu loader from a database or other source.
    """

    def __init__(self, app=None):
        """
        Arguments:
            app (flask.Flask): Flask app
        """

        if app:
            self.init_app(app)

        self._menu_funcs = {}

    def init_app(self, app):
        @app.context_processor
        def processors():
            return dict(
                menu=self,
            )

    def add_menu_items(self, menu_name):
        """
        Decorator for adding new menu items

        menu = MenuLoader()

        @menu.add_menu_items('topbar')
        def more_items():
            return [MenuItem(...),MenuItem(...)]
        """

        def real_decorator(function):
            if menu_name not in self._menu_funcs:
                self._menu_funcs[menu_name] = []
            self._menu_funcs[menu_name].append(function)
            return function

        return real_decorator

    def navbar_menu(self):
        return self.load_menu("topbar")

    def sidebar_menu(self):
        return self.load_menu("sidebar")

    def second_sidebar_menu(self):
        return self.load_menu("second_sidebar")

    def is_active(self, item):
        return item.url == (
            flask.request.script_root
            + flask.request.path
            + (
                f"?{flask.request.query_string.decode()}"
                if flask.request.query_string
                else ""
            )
        )

    def load_menu(self, menu_name):
        items = menu_from_yaml()
        if menu_name not in items:
            return None
        item = items[menu_name][0]["items"]
        title = items[menu_name][0]["title"]
        m = Menu(title=title)
        _create_menu_items(m, item)

        show_menu.send(
            flask.current_app._get_current_object(),
            loader=self,
            menu_name=menu_name,
            menu=m,
        )
        activated = m.activate_by_function(self.is_active, all=True)

        return m


_signals = Namespace()

show_menu = _signals.signal("show-menu")
""" 
show_menu signal is sent before rendering the menu. 
You can use it to add additional items to the menu.

Example:

    @show_menu.connect
    def _add_menu(app,menu_name,menu)
        if menu_name == 'admin_menu':
            menu.add_item(MenuItem(...))
"""

if signals.signals_available:

    @show_menu.connect
    def _add_menu(app, loader, menu_name, menu):
        # Add to menu all menuitems
        # from the menu loader functions
        # for menu_name
        if menu_name in loader._menu_funcs:
            funcs = loader._menu_funcs[menu_name]
            for f in funcs:
                menu_items = f()
                for item in menu_items:
                    menu.add_item(item)


def menu_from_yaml():
    menu_file = DEFAULT_MENUS_FILE
    menu = None

    # overwrrite the menu using the menus.yml in the instance settings
    if flask.has_app_context():
        instance_menu_file = os.path.join(flask.current_app.instance_path, "menus.yml")
        if os.path.exists(instance_menu_file):
            menu_file = instance_menu_file
        else:
            instance_menu_file = pathlib.Path(__file__).parent / "menus.yml"
            if os.path.exists(instance_menu_file):
                menu_file = instance_menu_file

    with open(menu_file) as f:
        menu = yaml.load(f, Loader=yaml.SafeLoader)

    return menu


def _create_menu_items(menu: Menu, items: dict, parent=None):
    """
    Recursively add dictionary of menu items to menu
    """
    for attrs in items:
        children = attrs.pop("items", [])
        title = attrs.pop("title", "Missing Title")
        link = attrs.pop("url", "#")
        badge = attrs.pop("badge", None)
        endpoint = attrs.pop("endpoint", None)
        help = attrs.pop("help", "")
        icon = attrs.pop("icon", None)
        form = attrs.pop("form", None)

        kind = MenuItem.TYPE_FORM if form else MenuItem.TYPE_LINK

        item_type = attrs.pop("item_type", kind)

        endpoint_kwargs = attrs.pop("endpoint_kwargs", "")
        if endpoint:
            try:
                if endpoint_kwargs:
                    if isinstance(endpoint_kwargs, str):
                        link = flask.url_for(endpoint) + "?" + endpoint_kwargs
                    elif isinstance(endpoint_kwargs, dict):
                        link = flask.url_for(endpoint, **endpoint_kwargs)
                    else:
                        print(
                            f"Unexpected endpoint_kwargs type: {type(endpoint_kwargs)} in MenuItem {title}"
                        )
                else:
                    link = flask.url_for(endpoint)

            except werkzeug.routing.BuildError as e:
                help += f"Error: {e}"
                if not badge:
                    badge = ["Soon", "danger"]

        elif link.startswith("/") and not link.startswith("//"):
            link = flask.request.script_root + link

        item = MenuItem(
            id_item=title,
            item_type=item_type,
            title=title,
            url=link,
            parent=parent,
            icon=icon,
            help=help,
            badge=badge,
            form=form,
        )

        menu.add_item(item)

        if children:
            _create_menu_items(menu, children, parent=item)


menu = MenuLoader()


def init_app(app):
    menu.init_app(app)



"""Contains all the constant values used in this module."""

THEME_BLUEPRINT_NAME = "theme"

THEME_URL_PREFIX = "/theme"

THEME_CONFIG_DEBUG = False
"""Print the theme default configurations at startup"""

BASE_TEMPLATE = "layouts/page.html"
"""The base template that all templates should extend. Defaults to 'layouts/page.html"""

BASE_IFRAME_TEMPLATE = 'layouts/iframe.html'
"""The base template for iframe. Defaults to 'layouts/iframe.html"""

THEME_ACCENT_COLOR = None
"""The color of the hyperlinks. Defaults to None."""

THEME_CONTROLBAR_LIGHT = False
"""Light theme for the control sidebar, the default is dark. Defaults to False."""

THEME_BRAND_HTML = "<b>EAGLE</b> Beta"
"""HTML version of the logo used on login pages and footer. Defaults to '<b>Admin</b>LTE 3'."""

THEME_LOGOUT_ENDPOINT = "" #"security.logout"
"""The name of the endpoint for user logout. Defaults to 'security.logout'."""

THEME_LOGIN_ENDPOINT = "" # "security.login"
"""The name of the endpoint for user login. Defaults to 'security.login'."""

THEME_REGISTRATION_ENDPOINT = "security.register"
"""The name of the endpoint for user registration. Defaults to 'security.register'."""

THEME_PROFILE_ENDPOINT = ""
"""The name of the endpoint for user registration. Defaults to ''."""

THEME_CHANGE_LANGUAGE_ENDPOINT = "translation.change_language"
"""The name of the endpoint for changing the language. Defaults to 'translation.change_language' """

THEME_BRAND_FAVICON = "data:image/x-icon"
"""The favicon url to show in the browser tab.  Defaults to 'data:image/x-icon' to disable on most browsers"""

THEME_JAVASCRIPT_TEMPLATE = "includes/javascript.html"
"""Javascript assets template, normally included in BASE_TEMPLATE.
The default template just includes the AdminLTE3 JavaScript files.
Set a new template if you would like to customize which JavaScript assets are
included on all pages. Defaults to 'includes/javascript.html'
"""

THEME_STYLESHEETS_TEMPLATE = "includes/stylesheets.html"
"""Stylesheet assets template, normally included in BASE_TEMPLATE.
The default template just includes the AdminLTE3 css files.
Set a new template if you would like to customize which CSS assets are
included on all pages. Defaults to 'includes/stylesheets.html'
"""

THEME_SITE_TITLE = "AdminLTE 3"
"""The title of the site. Defaults to 'AdminLTE 3'."""

THEME_META_TEMPLATE = "includes/meta.html"
"""Favicon template normally included in BASE_TEMPLATE. Defaults to 'includes/meta.html'"""

THEME_FAVICON_TEMPLATE = "includes/favicon.html"
"""Favicon template normally included in BASE_TEMPLATE. Defaults to 'includes/favicon.html'"""

THEME_TOPBAR_TEMPLATE = "includes/navigation.html"
"""Navigation top bar template which is normally included in BASE_TEMPLATE. Defaults to 'includes/navigation.html'"""

THEME_TOPBAR_USER_MENU_TEMPLATE = "includes/navigation/user_menu.html"
"""User panel template which is normally included in THEME_TOPBAR_TEMPLATE. Defaults to 'includes/navigation/user_menu.html'"""

THEME_TOPBAR_USER_MENU_DROPDOWN_TEMPLATE = "includes/navigation/user_menu_dropdown.html"
"""User panel dropdown template which is normally included in THEME_TOPBAR_TEMPLATE. Defaults to 'includes/navigation/user_menu_dropdown.html'"""

THEME_TOPBAR_NAV_TEMPLATE = "includes/navigation/nav.html"
"""Navigation Menu template which is normally included in THEME_TOPBAR_TEMPLATE. Defaults to 'includes/navigation/nav.html'"""

THEME_TOPBAR_SEARCH_TEMPLATE = "includes/navigation/search.html"
"""Topbat search which is normally included in THEME_TOPBAR_TEMPLATE. Defaults to 'includes/navigation/search.html'"""

THEME_TOPBAR_NOTIFICATIONS_TEMPLATE = "includes/navigation/notifications.html"
"""Topbar notifications template normally included in THEME_TOPBAR_TEMPLATE. Defaults to 'includes/navigation/notifications.html'"""

THEME_TOPBAR_USER_MENU_ENABLED = True
"""Enable the user menu panel on the top navigation bar. Defaults to True"""

THEME_ASSETS_ENABLED = True
"""Use compressed assets"""

THEME_ASSETS_PATH = None
"""Path to the source assets. Defaults to 'None'. If 'None', path is set to 'app.root_path/../assets'"""

THEME_SIDEBAR_ENABLED = True
"""Enable the main sidebar menu"""

THEME_PRELOADER_ENABLED = False
"""Enable preloading screen."""

THEME_BRAND_IMAGE_CLASS='brand-image img-circle elevation-3'
"""The css class for Brand Image"""

THEME_BRAND_IMAGE_STYLE='opacity: .8'
"""The css style for Brand Image"""

THEME_BRAND_IMAGE_URL = ''
"""URL for the Logo"""

THEME_SIDEBAR_TEMPLATE = "includes/sidebar.html"
"""Sidebar template which is normally included in BASE_TEMPLATE. Defaults to 'includes/sidebar.html'"""

THEME_SIDEBAR_NAV_TEMPLATE = "includes/sidebar/nav.html"
"""Navigation Menu template which is normally included in THEME_SIDEBAR_TEMPLATE. Defaults to 'includes/sidebar/nav.html'"""

THEME_SIDEBAR_USER_MENU_TEMPLATE = "includes/sidebar/user_menu.html"
"""User panel template which is normally included in THEME_SIDEBAR_TEMPLATE. Defaults to 'includes/sidebar/user_menu.html'"""

THEME_SIDEBAR_SEARCH_TEMPLATE = "includes/sidebar/search.html"
"""Sidebar search which is normally included in THEME_SIDEBAR_TEMPLATE. Defaults to 'includes/sidebar/search.html'"""

THEME_SIDEBAR_LOGO_TEMPLATE = "includes/sidebar/logo.html"
"""Brand Logo template which is normally included in THEME_SIDEBAR_TEMPLATE. Defaults to 'includes/sidebar/logo.html'"""

THEME_SIDEBAR_USER_MENU_ENABLED = True
"""Enable the user menu panel on the sidebar. Defaults to False"""

THEME_SIDEBAR_SEARCH_ENABLED = False
"""Enable the search input field on the sidebar. Defaults to False"""

THEME_CONTROLBAR_TEMPLATE = "includes/controlbar.html"
"""Control sidebar template which is normally included in BASE_TEMPLATE. Defaults to 'includes/controlbar.html'"""

THEME_CONTROLBAR_ENABLED = True
"""Enable the control sidebar. Defaults to False"""

THEME_FOOTER_TEMPLATE = "includes/footer.html"
"""Footer template which is normally included in BASE_TEMPLATE. Defaults to 'includes/footer.html'"""

THEME_BACK_TO_TOP_ENABLED = False
"""Turn on the back to top button. Defaults to False."""

THEME_ERRORS = False
"""Set to True to add the THEME_<error>_TEMPLATE error handlers to app"""

THEME_ERROR_BASE_TEMPLATE = "pages/error.html"
"""The base template that error pages extend"""

THEME_401_TEMPLATE = "pages/401.html"
"""The template used for 401 Unauthorized errors."""

THEME_403_TEMPLATE = "pages/403.html"
"""The template used for 403 Forbidden errors."""

THEME_404_TEMPLATE = "pages/404.html"
"""The template used for 404 Not Found errors."""

THEME_429_TEMPLATE = "pages/429.html"
"""The template used for 429 Too Many Requests errors."""

THEME_500_TEMPLATE = "pages/500.html"
"""The template used for 500 Internal Server Error errors."""

THEME_DARKMODE_SWITCH_ENABLED = True
"""Enable the darkmode switcher"""

THEME_FULLSCREEN_SWITCH_ENABLED = True
"""Enable the fullscreen switcher"""

THEME_STATIC_FOLDER = None
"""Override the theme's static folder path"""

THEME_TEMPLATE_FOLDER = None
"""Override the theme's template folder path"""

THEME_DEMO_ENABLED = False
"""Enable the controlbar theme customizer"""

THEME_LAYOUT_BOXED = False
"""Boxed Layout: use the class .layout-boxed to get a boxed layout that stretches only to 1250px. 
You cannot use both THEME_LAYOUT_BOXED and THEME_LAYOUT_FIXED_TOP_NAV 
or THEME_LAYOUT_FIXED_FOOTER at the same time. Anything else can be mixed together.
"""

THEME_LAYOUT_COLLAPSED_SIDEBAR = False
"""Collapsed Sidebar: use the class .sidebar-collapse to have a collapsed sidebar upon loading."""

THEME_LAYOUT_SIDEBAR_MINI = False
"""Sidebar Mini: use the .sidebar-mini class to show the sidebar when collapsed"""

THEME_LAYOUT_FIXED_FOOTER = True
"""Fixed Footer: use the class .layout-footer-fixed to get a fixed footer."""

THEME_LAYOUT_FIXED_SIDEBAR = True
"""Fixed Sidebar: adds the class .layout-fixed to get a fixed sidebar."""

THEME_LAYOUT_FIXED_TOP_NAV = True
"""Fixed Navbar: use the class .layout-navbar-fixed to get a fixed navbar."""

THEME_LAYOUT_TOP_NAV = False
"""Top Navigation: use the class .layout-top-nav to remove the sidebar and have your links at the top navbar."""


class ThemeColor:
    """Color styles."""

    PRIMARY = "primary"
    SECONDARY = "secondary"
    INFO = "info"
    SUCCESS = "success"
    WARNING = "warning"
    DANGER = "danger"

    WHITE = "white"
    BLACK = "black"
    GRAY_DARK = "gray-dark"
    GRAY = "gray"
    LIGHT = "light"

    INDIGO = "indigo"
    LIGHTBLUE = "lightblue"
    NAVY = "navy"
    PURPLE = "purple"
    FUCHSIA = "fuchsia"
    PINK = "pink"
    MAROON = "maroon"
    ORANGE = "orange"
    LIME = "lime"
    TEAL = "teal"
    OLIVE = "olive"

    GRADIENT_PRIMARY = "gradient-primary"
    GRADIENT_SUCCESS = "gradient-success"
    GRADIENT_DANGER = "gradient-danger"
    GRADIENT_WARNING = "gradient-warning"
    GRADIENT_INFO = "gradient-info"


__all__ = ("ThemeColor",)



import flask
import os
from . import filters
from . import constants


class Theme:
    def __init__(self, app=None) -> None:
        if app:
            self.init_app(app)

    def init_app(self, app: flask.Flask):

        for k in dir(constants):
            if k.startswith("THEME_"):
                app.config.setdefault(k, getattr(constants, k))

        bp = flask.Blueprint(
            app.config["THEME_BLUEPRINT_NAME"],
            __name__,
            static_folder=app.config["THEME_STATIC_FOLDER"] or "static",
            template_folder=app.config["THEME_TEMPLATE_FOLDER"] or "templates",
        )

        app.register_blueprint(bp, url_prefix=app.config["THEME_URL_PREFIX"])

        if app.config["THEME_ERRORS"]:
            set_theme_errors(app)

        for name in filters.__all__:
            app.jinja_env.filters.setdefault(name, getattr(filters, name))

        @app.context_processor
        def processors():
            return dict(
                ThemeColor=constants.ThemeColor,
            )

        try:
            from flask_gravatar import Gravatar

            Gravatar(app, default="mp")
        except ImportError as e:

            @app.template_filter("gravatar")
            def gravatar(email):
                return email

            app.logger.debug(f'Missing "flask_gravatar" package')


def set_theme_errors(app):

    # Register errors handlers.
    from .views import (
        unauthorized,
        insufficient_permissions,
        page_not_found,
        too_many_requests,
        internal_error,
    )

    app.register_error_handler(401, unauthorized)
    app.register_error_handler(403, insufficient_permissions)
    app.register_error_handler(404, page_not_found)
    app.register_error_handler(429, too_many_requests)
    app.register_error_handler(500, internal_error)



from contextlib import contextmanager
import blinker
from collections.abc import Iterable

@contextmanager
def captured_signals(app,signals=None):

    if not signals:
        signals = blinker.signal.__self__.values()
    recorded = []
    if not isinstance(signals,Iterable):
        signals = [signals]
    for s in signals:

        def record(sender, *args,**kwargs):
            recorded.append((s, sender, args, kwargs))
        s.connect(record, app)
    try:
        yield recorded
    finally:
        s.disconnect(record, app)

def init_app(app):
    if not app.testing:
        return


from setuptools import find_packages, setup
import os


def get_version():
    HERE = os.path.dirname(__file__)
    with open(os.path.join(HERE, NAME, "__version__.py")) as f:
        return f.read().strip()


NAME = "flask_theme_adminlte3"
VERSION = get_version()

def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join("..", path, filename))
    print(f"Package Files: {paths}")
    return paths


REQUIREMENTS = [
    "flask","pyyaml"
]
DEV_REQUIREMENTS = [
    "wheel","twine",
    "pytest", 
    "flask-debugtoolbar",
    "flask-assets",
    "jsmin",
    "cssmin",    
    "pre-commit",
    "flask-mkdocs"
]

setup(
    name=NAME,
    version=VERSION,
    packages=find_packages(exclude=['']),
    package_data={NAME: package_files(NAME)},
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    extras_require={
        "dev": DEV_REQUIREMENTS,
    }
)



# -*- coding: utf-8 -*-
import flask, click
from pathlib import Path
from . import theme, menu, assets

def create_app(config_file: Path = None,**overrides:flask.Config):
    """Create an application with custom configurations.

    Configurations passed into this function will override
    all other configurations.

    Args:
        config_file (Path, optional): Path to a config file. Relative
            paths will be relative to the instance_path. Defaults to None.
        overrides (dict, optional): dictionary of configuration value overrides.

    Raises:
        EnvironmentError: The configuration file does not exist
    """

    app = flask.Flask('theme')

    if config_file and app.config.from_pyfile(config_file):
        app.logger.debug(f"Loaded config_file from {config_file}")

    for k,v in overrides.items():
        app.config[k.upper()] = v

    theme.init_app(app)
    menu.init_app(app)
    assets.init_app(app)

    text = '\n'.join([f"{k}: {v}" for k,v in app.config.items() if k.startswith('THEME_')])

    app.logger.debug(text)

    @app.route('/')
    def home():
        return flask.render_template_string(
        """
        {% extends config.BASE_TEMPLATE %}
        {% block page_content %}
            Hello World
        {% endblock %}
        """)

    @app.cli.command('version',help=f"Print the curent version of {app.name}")
    def print_version():
        f"""Print the version of {app.name}"""
        click.echo(flask.current_app.config['VERSION'])

    return app



from flask import current_app, render_template


def unauthorized(e):
    """Error handler to show a 401.html page in case of a 401 error."""
    return render_template(current_app.config["THEME_401_TEMPLATE"]), 401


def insufficient_permissions(e):
    """Error handler to show a 403.html page in case of a 403 error."""
    return render_template(current_app.config["THEME_403_TEMPLATE"]), 403


def page_not_found(e):
    """Error handler to show a 404.html page in case of a 404 error."""
    return render_template(current_app.config["THEME_404_TEMPLATE"]), 404


def too_many_requests(e):
    """Error handler to show a 429.html page in case of a 429 error."""
    return render_template(current_app.config["THEME_429_TEMPLATE"]), 429


def internal_error(e):
    """Error handler to show a 500.html page in case of a 500 error."""
    return render_template(current_app.config["THEME_500_TEMPLATE"]), 500



"""
Theme

"""

from . import ext

theme = ext.Theme()

def init_app(app):

    from . import constants

    for k in dir(constants):
        if k.upper() == k:
            default = getattr(constants, k)
            if app.config['THEME_CONFIG_DEBUG'] == True:
                app.logger.debug(f'Setting Theme default {k} = {default}')
            app.config.setdefault(k, default)
        
    theme.init_app(app)
